@@ -1,22 +1,9 @@
-/*****************************************************************************
- * This file is part of Kiten, a KDE Japanese Reference Tool                 *
- * Copyright (C) 2006 Joseph Kerian <jkerian@gmail.com>                      *
- *                                                                           *
- * This library is free software; you can redistribute it and/or             *
- * modify it under the terms of the GNU Library General Public               *
- * License as published by the Free Software Foundation; either              *
- * version 2 of the License, or (at your option) any later version.          *
- *                                                                           *
- * This library is distributed in the hope that it will be useful,           *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
- * Library General Public License for more details.                          *
- *                                                                           *
- * You should have received a copy of the GNU Library General Public License *
- * along with this library; see the file COPYING.LIB.  If not, write to      *
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,      *
- * Boston, MA 02110-1301, USA.                                               *
- *****************************************************************************/
+/*
+    This file is part of Kiten, a KDE Japanese Reference Tool
+    SPDX-FileCopyrightText: 2006 Joseph Kerian <jkerian@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 /**
  * Future Plans:
@@ -40,6 +27,7 @@ ButtonGrid::ButtonGrid( QWidget *parent, RadicalFile *radicalInfo )
 : QWidget( parent )
 , CurrentMode( Selection )
 , m_radicalInfo( radicalInfo )
+, m_sortByFrequency( false )
 {
   if ( m_radicalInfo )
   {
@@ -53,6 +41,15 @@ ButtonGrid::~ButtonGrid()
 
 void ButtonGrid::buildRadicalButtons()
 {
+  if( layout() )
+  {
+    // Instead of iterating over our children and deleting them one by one, we
+    // simply reparent the previous layout to a temporary widget; when it goes
+    // out of scope, everything will be automatically deleted.
+    // Thanks to https://stackoverflow.com/a/10439207 for the tip!
+    QWidget().setLayout( layout() );
+  }
+
   //Setup the grid
   QGridLayout *grid = new QGridLayout( this );
 
@@ -70,43 +67,46 @@ void ButtonGrid::buildRadicalButtons()
   }
 
   //Get a list of radicals (organized by strokes)
-  QMultiMap<int, Radical> *radicalMap = m_radicalInfo->mapRadicalsByStrokes();
-  QMultiMap<int, Radical>::const_iterator it = radicalMap->constBegin();
+  QMultiMap<int, Radical> *radicalMap = m_radicalInfo->mapRadicalsByStrokes( number_of_radical_columns );
+  QList<int> radicalStrokeCounts = radicalMap->uniqueKeys();
   //Now create all the buttons
-  unsigned int last_column = 0;
-  int row_index = 1;
-  while( it != radicalMap->constEnd() )
+  foreach( int strokeCount, radicalStrokeCounts )
   {
-    //For each radical, figure out which slot it goes in (0-based column index)
-    unsigned int column_index = it.key() - 1;
-    if( column_index >= number_of_radical_columns )
+    //(0-based column index)
+    unsigned int column_index = strokeCount - 1;
+    int row_index = 1;
+
+    QList<Radical> radicals = radicalMap->values( strokeCount );
+    std::sort( radicals.begin(), radicals.end(),
+               m_sortByFrequency ? Radical::compareFrequencies : Radical::compareIndices );
+    foreach( const Radical &radical, radicals )
     {
-      column_index = number_of_radical_columns - 1;
+      //Make the button
+      RadicalButton *button = new RadicalButton( radical.toString(), this );
+      grid->addWidget( button, row_index++, column_index );
+      //Bind slots/signals for this button
+      connect( button, &RadicalButton::userClicked,
+                 this,   &ButtonGrid::radicalClicked );
+      connect(   this, &ButtonGrid::clearButtonSelections,
+               button,   &RadicalButton::resetButton );
+
+      //Add this button to our list
+      m_buttons.insert( radical.toString(), button );
     }
-
-    //If we're starting a new column, reset the row
-    if( last_column != column_index )
-    {
-      row_index = 1;
-    }
-
-    //Make the button
-    RadicalButton *button = new RadicalButton( *it, this );
-    grid->addWidget( button, row_index++, column_index );
-    //Bind slots/signals for this button
-    connect( button, &RadicalButton::userClicked,
-               this,   &ButtonGrid::radicalClicked );
-    connect(   this, &ButtonGrid::clearButtonSelections,
-             button,   &RadicalButton::resetButton );
-
-    //Add this button to our list
-    m_buttons.insert( *it, button );
-
-    last_column = column_index;
-    ++it;
   }
   delete radicalMap;
   setLayout( grid );
+
+  updateButtons();
+}
+
+void ButtonGrid::setSortByFrequency( bool enable )
+{
+  if( m_sortByFrequency != enable )
+  {
+    m_sortByFrequency = enable;
+    buildRadicalButtons();
+  }
 }
 
 void ButtonGrid::clearSelections()
@@ -144,14 +144,6 @@ void ButtonGrid::radicalClicked(  const QString &newrad
   }
 }
 
-void ButtonGrid::setFont( const QFont &font )
-{
-  foreach( QPushButton *button, m_buttons )
-  {
-    button->setFont( font );
-  }
-}
-
 void ButtonGrid::updateButtons()
 {
   if ( ! m_radicalInfo )
